package cases

import (
	"spaceofcheng/services/models/model"
)

// GetKeyList 获取钥匙列表
func (c *CaseController) GetKeyList() {
	caseids := c.GetString("caseid")
	if caseids == "" {
		cases := c.Context.Get("cases").([]model.SysUserCase)
		caseids = c.GetCaseIDs(cases)
	}
	page, _ := c.GetInt("page")
	pageSize, _ := c.GetInt("pagesize")
	num := c.GetString("num")

	if page < 1 {
		page = 1
	}

	list, err := c.dao.GetKeysByCase(caseids, num, page, pageSize)
	if err != nil {
		c.ResponseError(err)
	}
	c.ResponseJSON(list)
}

// AddKeys 新增钥匙
func (c *CaseController) AddKeys() {
	caseid := c.GetString("caseid")
	orgid := c.GetString("orgid")
	beginnum, _ := c.GetInt("beginnum")
	endnum, _ := c.GetInt("endnum")

	err := c.dao.AddKeys(orgid, caseid, beginnum, endnum)
	if err != nil {
		c.ResponseError(err)
	}
	c.ResponseJSON("操作成功！")
}

// UnLockKey 解锁
func (c *CaseController) UnLockKey() {
	keyid := c.GetString(":keyid")
	err := c.dao.UnLockKey(keyid)
	if err != nil {
		c.ResponseError(err)
	}
	c.ResponseJSON("解锁成功！")
}

// LockKey 绑定钥匙
func (c *CaseController) LockKey() {
	keyId := c.GetString(":keyid")
	customerid := c.GetString(":customerid")
	customername := c.GetString(":customername")
	err := c.dao.LockKey(keyId, customerid, customername)
	if err != nil {
		c.ResponseError(err)
	}
	c.ResponseJSON("绑定成功！")

}

// LockKeyByPhone 手机号绑定钥匙
func (c *CaseController) LockKeyByPhone() {
	phone := c.GetString(":phone")
	keyId := c.GetString(":keyid")
	err := c.dao.LockKeyByPhone(phone, keyId)
	if err != nil {
		c.ResponseError(err)
	}
	c.ResponseJSON("绑定成功！")
}

// LockKeyVerify 二维码核销绑定钥匙
func (c *CaseController) LockKeyVerify() {
	customerId := c.GetString(":customerid")
	keyId := c.GetString(":keyid")
	err := c.dao.LockKeyVerify(keyId, customerId)
	if err != nil {
		c.ResponseError(err)
	}
	c.ResponseJSON("绑定成功！")

}

// DelKey 删除
func (c *CaseController) DelKey() {
	keyid := c.GetString(":keyid")
	err := c.dao.DelKey(keyid)
	if err != nil {
		c.ResponseError(err)
	}
	c.ResponseJSON("删除成功！")
}
