package luckdraw

import (
	"cdkj-check/helper"
	"errors"
	"net/http"
	"spaceofcheng/services/controllers"
	"spaceofcheng/services/models/model"
	"spaceofcheng/services/service/luckdraw"
	"spaceofcheng/services/utils"
	"strconv"
)

// LuckdrawController 应用
type LuckdrawController struct {
	dao *luckdraw.LuckdrawServ
	controllers.BaseController
}

// Constructor 初始化 Controller
// @Title Constructor
// @Description 初始化 Controller, 系统自动调用
func (c *LuckdrawController) Constructor() {
	c.dao = luckdraw.NewLuckdrawServ(c.Context)
	org := c.Context.Get("org").(model.SysOrg)
}

// GetUserByCode 根据code获取用户信息
func (c *LuckDrawController) GetUserByCode() {
	custID := 0
	encodeUser, _ := c.GetInt("from")
	if encodeUser != 0 {
		encodeUser = helper.IntXOR(encodeUser, 97)
		enStr := strconv.Itoa(encodeUser)
		enStr = enStr[:len(enStr)-3]

		custID, _ = strconv.Atoi(enStr)
	}

	userRaw := c.Context.Get("customer")
	if userRaw == nil {
		c.ResponseError(errors.New("系统内部错误"), http.StatusInternalServerError)
	}

	user := userRaw.(model.TaCustomer)
	luckdrawid, _ := c.GetInt("luckdrawid")
	record, err := c.dao.GetUserLuckDrawByLuckDraw(user.Id, luckdrawid)
	if err != nil {
		utils.LogError("获取用户抽奖信息失败: " + err.Error())
		c.ResponseError(errors.New("获取用户抽奖信息失败"))
	}
	var detail *model.TaPrizeDetail
	if record != nil {
		detail, err = c.dao.GetDetailByRecord(record.Id)
		if err != nil {
			utils.LogError("获取用户抽奖信息失败: " + err.Error())
			c.ResponseError(errors.New("获取用户抽奖信息失败"))
		}
	}

	c.ResponseJSON(map[string]interface{}{
		"user":   user,
		"record": record,
		"detail": detail,
	})
}

// GetLuckDraw 获取抽奖信息
func (c *LuckDrawController) GetLuckDraw() {
	id, _ := c.GetInt(":id")
	luckdraw, err := c.dao.GetLuckDraw(id)
	if err != nil {
		utils.LogError("获取抽奖信息失败: " + err.Error())
		c.ResponseError(errors.New("获取抽奖信息失败"))
	}
	c.ResponseJSON(luckdraw)
}

// LuckDraw 抽奖
func (c *LuckDrawController) LuckDraw() {
	id, _ := c.GetInt(":id")
	userRaw := c.Context.Get("customer")
	if userRaw == nil {
		c.ResponseError(errors.New("系统内部错误"), http.StatusInternalServerError)
	}

	user := userRaw.(model.TaCustomer)

	prize, detail, err := c.dao.LuckDraw(id, user)
	if err != nil {
		c.ResponseError(err)
	}
	c.ResponseJSON(map[string]interface{}{
		"prize":  prize,
		"detail": detail,
	})
}

// GetRecordByLuckDraw 获取抽奖记录
func (c *LuckDrawController) GetRecordByLuckDraw() {
	id := c.GetString("id")
	records, err := c.dao.GetRecordByLuckDraw(id)
	if err != nil {
		utils.LogError("获取抽奖记录失败: " + err.Error())
		c.ResponseError(errors.New("获取抽奖记录失败"))
	}
	c.ResponseJSON(records)
}

// GetUserLuckDraw 获取用户的抽奖记录
func (c *LuckDrawController) GetUserLuckDraw() {
	userid := c.GetString("userid")
	records, err := c.dao.GetUserLuckDraw(userid)
	if err != nil {
		utils.LogError("获取用户抽奖记录失败: " + err.Error())
		c.ResponseError(errors.New("获取用户抽奖记录失败"))
	}
	c.ResponseJSON(records)
}

// GetRecordByID 根据ID获取中奖记录信息
func (c *LuckDrawController) GetRecordByID() {
	id, _ := c.GetInt(":id")
	record, err := c.dao.GetRecordByID(id)
	if err != nil {
		utils.LogError("获取中奖记录失败: " + err.Error())
		c.ResponseError(errors.New("获取中奖记录失败"))
	}

	caseAddress, err := c.dao.GetCaseAddressByLuckDarw(record.LuckdrawId)
	if err != nil {
		utils.LogError("获取案场信息失败: " + err.Error())
		c.ResponseError(errors.New("获取案场信息失败"))
	}

	if caseAddress == "" {
		c.ResponseError(errors.New("无案场, 或案场地址没有维护"))
	}

	prize, err := c.dao.GetPrizeByID(record.PrizeId)
	if err != nil {
		utils.LogError("校验奖品记录失败: " + err.Error())
		c.ResponseError(errors.New("校验奖品记录失败"))
	}

	c.ResponseJSON(map[string]interface{}{
		"record":      record,
		"prize":       prize,
		"caseAddress": caseAddress,
	})
}

func (c *LuckDrawController) SaveShareRecord() {
	errIgnore := "error-ignore"

	id := c.GetString(":id")

	fromID := c.GetString("from")
	if fromID == "" {
		c.ResponseData(
			errIgnore,
			"没有分享人ID",
			http.StatusBadRequest,
		)
	}

	from, err := c.bodydao.GetCustomerByID(fromID)
	if err != nil {
		utils.LogError("获取分享人信息失败: " + err.Error())
		c.ResponseData(
			errIgnore,
			"获取分享人信息失败",
			http.StatusBadRequest,
		)
	}

	if from == nil || from.CustomerId == "" {
		utils.LogError("分享人信息不存在: " + err.Error())
		c.ResponseData(
			errIgnore,
			"分享人信息不存在",
			http.StatusBadRequest,
		)
	}

	current = c.Context.Get("customer")

	// 自己分享给自己
	if from.CustomerId == current.CustomerId {
		c.ResponseData(
			errIgnore,
			"本人分享内容无效",
			http.StatusBadRequest,
		)
	}

	luckdraw, err := c.dao.GetLuckDrawByID(id)
	if err != nil {
		utils.LogError("获取抽奖信息失败: " + err.Error())
		c.ResponseData(
			errIgnore,
			"获取抽奖信息失败",
			http.StatusBadRequest,
		)
	}

	if luckdraw == nil {
		utils.LogError("获取抽奖信息失败: " + err.Error())
		c.ResponseData(
			errIgnore,
			"没有找到 id ("+id+") 对应的抽奖信息",
			http.StatusBadRequest,
		)
	}

	existsRec, err := c.dao.GetLuckDrawShareData(from.Id, current.Id, luckdraw.Id, luckdraw.CaseId)
	if err != nil {
		utils.LogError("获取已有分享信息失败: " + err.Error())
		c.ResponseData(
			errIgnore,
			"获取已有分享信息失败",
			http.StatusBadRequest,
		)
	}

	if existsRec != nil && len(existsRec) > 0 {
		c.ResponseData(
			errIgnore,
			"已分享记录, 再次分享无效",
			http.StatusBadRequest,
		)
	}

	shareDt := model.TaShareLuckyRecord{
		FromCustomerId:     from.CustomerId,
		FromCustomerName:   from.CustomerName,
		FromCustomerWxname: from.Name,
		FromCustomerTel:    from.Phone,
		ToCustomerId:       current.CustomerId,
		ToCustomerName:     current.CustomerName,
		ToCustomerWxname:   current.Name,
		ToCustomerTel:      current.Phone,
		CaseId:             luckdraw.CaseId,
		LuckydrawId:        luckdraw.Id,
		LuckydrawName:      luckdraw.Name,
	}

	if err := c.dao.SaveLuckDrawShareData(&shareDt); err != nil {
		utils.LogError("保存分享记录失败: " + err.Error())
		c.ResponseData(
			errIgnore,
			"保存分享记录失败",
			http.StatusBadRequest,
		)
	}

	c.ResponseJSON("ok")
}
