package statistics

import (
	"errors"
	"spaceofcheng/services/models/model"
	"spaceofcheng/services/utils"
	"strconv"
)

// CardCouponStatistics 获取卡券统计数据
func (c *StatisticsController) CardCouponStatistics() {
	page, _ := c.GetInt("page")
	pagesize, _ := c.GetInt("pagesize")
	caseids := c.GetString("caseid")
	name := c.GetString("name")
	ctype := c.GetString("ctype")

	if caseids == "" {
		cases := c.Context.Get("cases").([]model.SysUserCase)
		caseids = c.GetCaseIDs(cases)
	}
	list, err := c.serv.CardCouponStatistics(caseids, ctype, name, page, pagesize)
	if err != nil {
		c.ResponseError(err)
	}

	c.ResponseJSON(list)
}

// CardCouponStatisticsExcel 获取卡券统计数据
func (c *StatisticsController) CardCouponStatisticsExcel() {
	caseids := c.GetString("caseid")
	name := c.GetString("name")
	ctype := c.GetString("ctype")

	if caseids == "" {
		cases := c.Context.Get("cases").([]model.SysUserCase)
		caseids = c.GetCaseIDs(cases)
	}

	list, err := c.serv.CardCouponStatisticsExcel(caseids, ctype, name)
	if err != nil {
		c.ResponseError(err)
	}

	excel, err := utils.NewTinyXLSXEngine()
	if err != nil {
		utils.LogError("初始化Excel服务失败: " + err.Error())
		c.ResponseError(errors.New("初始化Excel服务失败"))
	}

	excel.SetCell(excel.InsertRow(), []string{
		"案场",
		"卡券名称",
		"卡券类型",
		"发送类型",
		"价格",
		"领取数",
		"实际使用数",
		"总费用",
	})

	for _, item := range list {
		row := excel.InsertRow()
		sendtype := ""
		switch item.SendType {
		case "system":
			sendtype = "系统"
			break
		case "case":
			sendtype = "案场"
			break
		case "channel":
			sendtype = "渠道"
			break
		}
		price, err := strconv.ParseFloat(item.Price, 64)
		if err != nil {
			c.ResponseError(errors.New("金额类型不正确！"))
		}
		amount := price * float64(item.UsedCount)

		excel.SetCell(row, []string{
			item.CaseName,
			item.CName,
			item.TypeName,
			sendtype,
			item.Price,
			strconv.Itoa(item.SentCount),
			strconv.Itoa(item.UsedCount),
			strconv.FormatFloat(amount, 'f', -1, 64),
		})
	}
	c.SaveToExcel("卡券统计列表.xlsx", excel)
}

// CardCouponUsedStatistics 获取卡券使用统计数据
func (c *StatisticsController) CardCouponUsedStatistics() {
	page, _ := c.GetInt("page")
	pagesize, _ := c.GetInt("pagesize")
	caseids := c.GetString("caseid")
	name := c.GetString("name")
	receivetype := c.GetString("receivetype")
	tel := c.GetString("tel")
	begindate := c.GetString("begindate")
	enddate := c.GetString("enddate")
	status := c.GetString("status")

	if caseids == "" {
		cases := c.Context.Get("cases").([]model.SysUserCase)
		caseids = c.GetCaseIDs(cases)
	}
	list, err := c.serv.CardCouponUsedStatistics(caseids, tel, name, receivetype, begindate, enddate, status, page, pagesize)
	if err != nil {
		c.ResponseError(err)
	}

	c.ResponseJSON(list)
}

// CardCouponUsedStatisticsExcel 卡券使用导出
func (c *StatisticsController) CardCouponUsedStatisticsExcel() {
	caseids := c.GetString("caseid")
	name := c.GetString("name")
	receivetype := c.GetString("receivetype")
	tel := c.GetString("tel")
	begindate := c.GetString("begindate")
	enddate := c.GetString("enddate")
	status := c.GetString("status")

	if caseids == "" {
		cases := c.Context.Get("cases").([]model.SysUserCase)
		caseids = c.GetCaseIDs(cases)
	}
	list, err := c.serv.CardCouponUsedStatisticsExcel(caseids, tel, name, receivetype, begindate, enddate, status)
	if err != nil {
		c.ResponseError(err)
	}

	excel, err := utils.NewTinyXLSXEngine()
	if err != nil {
		utils.LogError("初始化Excel服务失败: " + err.Error())
		c.ResponseError(errors.New("初始化Excel服务失败"))
	}

	excel.SetCell(excel.InsertRow(), []string{
		"案场",
		"卡券名称",
		"使用商品",
		"获取方式",
		"微信昵称",
		"手机号",
		"有效期",
		"获取时间",
		"使用时间",
		"状态",
	})

	for _, item := range list {
		row := excel.InsertRow()
		receivetype := ""
		switch item.ReceivingType {
		case "event":
			receivetype = "系统"
			break
		case "case":
			receivetype = "案场"
			break
		case "channel":
			receivetype = "渠道"
			break
		case "give":
			receivetype = "后台赠送"
			break
		}

		verifystatus := ""
		switch item.VerifyStatus {
		case "useable":
			verifystatus = "未使用"
			break
		case "used":
			verifystatus = "已使用"
			break
		case "late":
			verifystatus = "逾期核销"
			break
		case "expire":
			verifystatus = "已失效"
			break
		}

		excel.SetCell(row, []string{
			item.CaseName,
			item.CardCouponName,
			item.TargetName,
			receivetype,
			item.CustomerName,
			item.Phone,
			item.EndDate.Format("2006-01-02 15:04:05"),
			item.ReceiveDate.Format("2006-01-02 15:04:05"),
			item.UsedDate.Format("2006-01-02 15:04:05"),
			verifystatus,
		})
	}
	c.SaveToExcel("卡券使用列表.xlsx", excel)
}

// CardCouponVerifyStatistics 获取卡券核销统计数据
func (c *StatisticsController) CardCouponVerifyStatistics() {
	page, _ := c.GetInt("page")
	pagesize, _ := c.GetInt("pagesize")
	caseids := c.GetString("caseid")
	name := c.GetString("name")
	tel := c.GetString("tel")
	status := c.GetString("status")

	if caseids == "" {
		cases := c.Context.Get("cases").([]model.SysUserCase)
		caseids = c.GetCaseIDs(cases)
	}
	list, err := c.serv.CardCouponVerifyStatistics(caseids, tel, name, status, page, pagesize)
	if err != nil {
		c.ResponseError(err)
	}

	c.ResponseJSON(list)
}

// CardCouponVerifyStatisticsExcel 卡券核销统计导出
func (c *StatisticsController) CardCouponVerifyStatisticsExcel() {
	caseids := c.GetString("caseid")
	name := c.GetString("name")
	tel := c.GetString("tel")
	status := c.GetString("status")

	if caseids == "" {
		cases := c.Context.Get("cases").([]model.SysUserCase)
		caseids = c.GetCaseIDs(cases)
	}
	list, err := c.serv.CardCouponVerifyStatisticsExcel(caseids, tel, name, status)
	if err != nil {
		c.ResponseError(err)
	}

	excel, err := utils.NewTinyXLSXEngine()
	if err != nil {
		utils.LogError("初始化Excel服务失败: " + err.Error())
		c.ResponseError(errors.New("初始化Excel服务失败"))
	}

	excel.SetCell(excel.InsertRow(), []string{
		"案场",
		"课程名称",
		"课程时间",
		"姓名",
		"微信昵称",
		"手机号",
		"下单时间",
		"核销时间",
		"状态",
	})

	for _, item := range list {
		row := excel.InsertRow()
		verifystatus := ""
		switch item.Status {
		case "useable":
			verifystatus = "未使用"
			break
		case "used":
			verifystatus = "已使用"
			break
		case "late":
			verifystatus = "逾期核销"
			break
		}

		verifydate := ""
		if !item.VerifyDate.IsZero() {
			verifydate = item.VerifyDate.Format("2006-01-02 15:04:05")
		}

		excel.SetCell(row, []string{
			item.CaseName,
			item.CourseName,
			item.BeginDate.Format("2006-01-02 15:04:05") + "~" + item.EndDate.Format("2006-01-02 15:04:05"),
			item.Name,
			item.CustomerName,
			item.Phone,
			item.CreateDate.Format("2006-01-02 15:04:05"),
			verifydate,
			verifystatus,
		})
	}
	c.SaveToExcel("卡券核销列表.xlsx", excel)
}
