package vipcard

import (
	"errors"
	"spaceofcheng/services/controllers"
	"spaceofcheng/services/models"
	"spaceofcheng/services/models/model"
	"spaceofcheng/services/service/vipcard"
	"time"
	"spaceofcheng/services/utils"
)

// CaseController 信息
type VipcardController struct {
	dao *vipcard.VipcardServ
	controllers.BaseController
}

// Constructor 初始化 Controller
// @Title Constructor
// @Description 初始化 Controller, 系统自动调用
func (c *VipcardController) Constructor() {
	c.dao = vipcard.NewVipcardServ(c.Context)
}

// GetVipList 获取VIP卡列表
func (c *VipcardController) GetVipList() {
	caseids := c.GetString("caseid")
	if caseids == "" {
		cases := c.Context.Get("cases").([]model.SysUserCase)
		caseids = c.GetCaseIDs(cases)
	}

	page, _ := c.GetInt("page")
	pageSize, _ := c.GetInt("pagesize")

	cardNo := c.GetString("cardNo")
	sellerName := c.GetString("sellerName")
	userName := c.GetString("userName")

	list, err := c.dao.GetVipChild(caseids, cardNo, sellerName, userName, page, pageSize)
	if err != nil {
		c.ResponseError(err)
	}
	c.ResponseJSON(list)
}

// AddVipCard 新增VIP卡
func (c *VipcardController) AddVipCard() {
	vipcard := model.TaVipCard{}
	if err := c.ParseForm(&vipcard); err != nil {
		c.ResponseError(err)
	}
	newVipCard, err := c.dao.AddVipCard(vipcard)
	if err != nil {
		c.ResponseError(err)
	}
	c.ResponseJSON(newVipCard)

}

// userCharge 用户激活vip卡
func (c *VipcardController) UserCharge() {
	vipCode := c.GetString(":vipCardChildCode")
	tel := c.GetString("tel")
	var createUser = c.Context.Get("user").(model.SysUser)
	vipChild, err := c.dao.GetVipChildByCode(vipCode)
	if err != nil {
		c.ResponseError(err)
	}

	customer, err := c.dao.FindCustomerByTel(tel)
	if err != nil {
		c.ResponseError(err)
	}

	if customer == nil {
		c.ResponseError(errors.New("没有查询到用户数据！"))
	}

	salesid := c.GetString("salesid")
	salesname := c.GetString("salesname")

	vipChild.CustomerId = customer.CustomerId
	vipChild.CustomerTel = tel
	vipChild.ActiveDate = time.Now()
	vipChild.Status = models.STATUS_NORMAL
	vipChild.SalesId = salesid
	vipChild.SalesName = salesname
	err = c.dao.UpdateVipChild(vipChild)
	if err != nil {
		c.ResponseError(err)
	}
	err = c.dao.CustomerCharge(vipChild, createUser)
	if err != nil {
		c.ResponseError(err)
	}
	c.ResponseJSON("激活成功")

}

// GetVipByCode 根据code获取Vip卡信息
func (c *VipcardController) GetVipByCode() {
	code := c.GetString(":code")
	vipChild, err := c.dao.GetVipChildByCode(code)
	if err != nil {
		c.ResponseError(err)
	}
	c.ResponseJSON(vipChild)
}


// GetVipListExcel 导出VIP 列表
func (c *VipcardController) GetVipListExcel(){

	caseids := c.GetString("caseid")
	if caseids == "" {
		cases := c.Context.Get("cases").([]model.SysUserCase)
		caseids = c.GetCaseIDs(cases)
	}

	cardNo := c.GetString("cardNo")
	sellerName := c.GetString("sellerName")
	userName := c.GetString("userName")

	list, err := c.dao.GetVipChildExcel(caseids, cardNo, sellerName, userName)
	if err != nil {
		c.ResponseError(err)
	}

	excel, err := utils.NewTinyXLSXEngine()
	if err != nil {
		utils.LogError("初始化Excel服务失败: " + err.Error())
		c.ResponseError(errors.New("初始化Excel服务失败"))
	}

	// 创建表头
	excel.SetCell(excel.InsertRow(), []string{
		"卡号",
		"价格",
		"状态",
		"销售",
		"案场",
		"用户",
		"激活时间",
		"创建时间",
	})

	// 遍历数据
	for _, item := range list {
		row := excel.InsertRow()
		createDateToString := item.CreateDate.Format("2006-01-02 15:04:05")
		activeDateToString := item.ActiveDate.Format("2006-01-02 15:04:05")
		statusToString := ""
		if item.Status == 1 {
			statusToString = "已激活"
		}else{
			statusToString = "未激活"
		}

		excel.SetCell(row, []string{
			item.VipCardChildCode,
			item.Amount,
			statusToString,
			item.SalesName,
			item.CaseName,
			item.CustomerName,
			activeDateToString,
			createDateToString,
		})
	}
	c.SaveToExcel("VIP卡列表.xlsx", excel)

}




