package cases

import (
	"spaceofcheng/services/models"
	"spaceofcheng/services/models/model"
	"spaceofcheng/services/utils"
	"strconv"
	"strings"
	"time"

	"github.com/go-xorm/xorm"
)

// 时间范围
const (
	ALL        = -1 // 全部
	TODAY      = 0  // 今天
	THIS_WEEK  = 1  // 本周
	THIS_MONTH = 2  // 本月
	THIS_YEAR  = 3  // 今年
)

// 星期
const (
	MONDAY    = 1 // 星期一
	TUESDAY   = 2 // 星期二
	WEDNESDAY = 3 // 星期三
	THURSDAY  = 4 // 星期四
	FRIDAY    = 5 // 星期五
	SATURDAY  = 6 // 星期六
	SUNDAY    = 7 // 星期天
)

// CaseDAO 当前数据库操作对象
type SigninDAO struct {
	ctx *utils.Context
	db  *xorm.Session
}

// NewCaseDAO New Inst
func NewSigninDAO(ctx *utils.Context) *SigninDAO {
	return &SigninDAO{
		ctx: ctx,
		db:  ctx.DB,
	}
}

// SigninInfo 案场
type SigninInfo struct {
	model.TaCheckinRecord `xorm:"extends"`
	UserId                string
}

/**
	 GetSigninWhere 根据条件查询签到记录
			 条件:
					 selectType: 今天,本周,本月,今年,全部
					 caseid: 案场ID

		ALL 				= -1 // 全部
		TODAY 			= 0 // 今天
		THIS_WEEK 	= 1 // 本周
		THIS_MONTH 	= 2 // 本月
		THIS_YEAR  	= 3 // 今年


 **/
func (m *SigninDAO) GetSigninWhere(selectType int, caseid string, page, pageSize int) ([]SigninInfo, error) {
	var signin []SigninInfo
	var sql string
	switch selectType {
	case ALL:
		sql = `select tcr.*,user_id from ta_checkin_record tcr LEFT JOIN ta_customer tc on tcr.customer_id = tc.customer_id where tcr.case_id in ('` +
			strings.Replace(caseid, ",", "','", -1) + `') ORDER BY tcr.create_date DESC limit ` + strconv.Itoa((page-1)*pageSize) + `, ` + strconv.Itoa(pageSize)
	case TODAY:
		sql = `select tcr.*,user_id from ta_checkin_record tcr LEFT JOIN ta_customer tc on tcr.customer_id = tc.customer_id where to_days(tcr.create_date) = to_days(now()) and tcr.case_id in ('` +
			strings.Replace(caseid, ",", "','", -1) + `') ORDER BY tcr.create_date DESC limit ` + strconv.Itoa((page-1)*pageSize) + `, ` + strconv.Itoa(pageSize)
	case THIS_WEEK:
		sql = `select tcr.*,user_id from ta_checkin_record tcr LEFT JOIN ta_customer tc on tcr.customer_id = tc.customer_id WHERE YEARWEEK(date_format(tcr.create_date,'%Y-%m-%d')) = YEARWEEK(now())  and tcr.case_id in ('` +
			strings.Replace(caseid, ",", "','", -1) + `') ORDER BY tcr.create_date DESC limit ` + strconv.Itoa((page-1)*pageSize) + `, ` + strconv.Itoa(pageSize)
	case THIS_MONTH:
		sql = `select tcr.*,user_id from ta_checkin_record tcr LEFT JOIN ta_customer tc on tcr.customer_id = tc.customer_id WHERE DATE_FORMAT(tcr.create_date,'%Y%m') = DATE_FORMAT(CURDATE(),'%Y%m') and tcr.case_id in ('` +
			strings.Replace(caseid, ",", "','", -1) + `') ORDER BY tcr.create_date DESC limit ` + strconv.Itoa((page-1)*pageSize) + `, ` + strconv.Itoa(pageSize)
	case THIS_YEAR:
		sql = `select tcr.*,user_id from ta_checkin_record tcr LEFT JOIN ta_customer tc on tcr.customer_id = tc.customer_id where YEAR(tcr.create_date)=YEAR(NOW()) and tcr.case_id in ('` +
			strings.Replace(caseid, ",", "','", -1) + `') ORDER BY tcr.create_date DESC limit ` + strconv.Itoa((page-1)*pageSize) + `, ` + strconv.Itoa(pageSize)
	}

	err := m.db.Sql(sql).Find(&signin)
	// for index := 0; 0 < len(signin); index++ {
	// 	println("用户编号:"+signin[index].UserId+"-")
	// 	if signin[index].UserId == "" {
	// 		signin[index].UserId = "前台用户"
	// 	} else {
	// 		signin[index].UserId = "后台用户"
	// 	}
	// }
	return signin, err
}

// AddSignin 添加签到记录
func (m *SigninDAO) AddSignin(signin model.TaCheckinRecord) (model.TaCheckinRecord, error) {
	signin.CheckinId = utils.GetGUID()
	signin.CreateDate = time.Now()
	signin.Status = models.STATUS_NORMAL
	_, err := m.db.Insert(signin)
	return signin, err
}

// ValidCheckIn 校验签到
func (m *SigninDAO) ValidCheckIn(caseID, openID, day string) (*model.TaCheckinRecord, error) {
	cs := new(model.TaCheckinRecord)
	has, err := m.db.Where("case_id=?", caseID).
		And("open_id=?", openID).
		And("DATE_FORMAT(create_date, '%Y-%m-%d')=?", day).Get(cs)

	if !has {
		return nil, err
	}

	return cs, nil
}

/**
	 GetSigninExcel 根据条件查询签到记录
			 条件:
					 selectType: 今天,本周,本月,今年,全部
					 caseid: 案场ID

		ALL 				= -1 // 全部
		TODAY 			= 0 // 今天
		THIS_WEEK 	= 1 // 本周
		THIS_MONTH 	= 2 // 本月
		THIS_YEAR  	= 3 // 今年


 **/
func (m *SigninDAO) GetSigninExcel(selectType int, caseid string) ([]SigninInfo, error) {
	var signin []SigninInfo
	var sql string
	switch selectType {
	case ALL:
		sql = `select tcr.*,user_id from ta_checkin_record tcr LEFT JOIN ta_customer tc on tcr.customer_id = tc.customer_id where tcr.case_id in ('` +
			strings.Replace(caseid, ",", "','", -1) + `') ORDER BY tcr.create_date DESC`
	case TODAY:
		sql = `select tcr.*,user_id from ta_checkin_record tcr LEFT JOIN ta_customer tc on tcr.customer_id = tc.customer_id where to_days(tcr.create_date) = to_days(now()) and tcr.case_id in ('` +
			strings.Replace(caseid, ",", "','", -1) + `') ORDER BY tcr.create_date DESC`
	case THIS_WEEK:
		sql = `select tcr.*,user_id from ta_checkin_record tcr LEFT JOIN ta_customer tc on tcr.customer_id = tc.customer_id WHERE YEARWEEK(date_format(tcr.create_date,'%Y-%m-%d')) = YEARWEEK(now())  and tcr.case_id in ('` +
			strings.Replace(caseid, ",", "','", -1) + `') ORDER BY tcr.create_date DESC`
	case THIS_MONTH:
		sql = `select tcr.*,user_id from ta_checkin_record tcr LEFT JOIN ta_customer tc on tcr.customer_id = tc.customer_id WHERE DATE_FORMAT(tcr.create_date,'%Y%m') = DATE_FORMAT(CURDATE(),'%Y%m') and tcr.case_id in ('` +
			strings.Replace(caseid, ",", "','", -1) + `') ORDER BY tcr.create_date DESC`
	case THIS_YEAR:
		sql = `select tcr.*,user_id from ta_checkin_record tcr LEFT JOIN ta_customer tc on tcr.customer_id = tc.customer_id where YEAR(tcr.create_date)=YEAR(NOW()) and tcr.case_id in ('` +
			strings.Replace(caseid, ",", "','", -1) + `') ORDER BY tcr.create_date DESC`
	}

	err := m.db.Sql(sql).Find(&signin)
	return signin, err
}
