package goods

import (
	"errors"
	"spaceofcheng/services/models"
	"spaceofcheng/services/models/model"
	"spaceofcheng/services/utils"
	"strconv"
	"strings"
	"time"

	"github.com/yl10/kit/guid"
)

const (
	USERTPE_SALES     = "sales"
	USERTYPE_CUSTOMER = "customer"
)

// GetOrdersByCases 获取案场订单列表
func (m *GoodsDAO) GetOrdersByCases() error {
	return nil
}

// SaveOrders 下单
func (m *GoodsDAO) SaveOrders(order *model.TaGoodsOrders) error {
	if order.UserId == "" {
		return errors.New("没有下单人信息")
	}

	caseID := order.CaseId
	if caseID == "" {
		return errors.New("没有指定案场")
	}

	caseInfo, err := m.getCaseByID(caseID)
	if err != nil {
		return err
	}

	// order.OrdersId = guid.NewGUIDString()
	order.OrgId = caseInfo.OrgId
	order.CreateDate = time.Now().Local()
	order.Status = models.STATUS_NORMAL
	order.IsPay = models.BOOL_TRUE // 默认已支付
	order.CouponAmount = "0"

	// 当前订单号的随机方式 = 时间 + 个人ID
	ordersno := "G-" + time.Now().Local().Format("20060102150405") + "-" + strings.Join(utils.GUIID2IntString(order.UserId), "")
	order.OrdersNo = string([]byte(ordersno)[:32])
	if _, err := m.db.Insert(order); err != nil {
		utils.LogError("保存订单主记录失败: " + err.Error())

		return errors.New("保存订单主记录失败")
	}
	return nil
}

// SaveOrdersDetail 保存订单明细
func (m *GoodsDAO) SaveOrdersDetail(list []model.TaGoodsOrdersDetail, ordersID string) error {
	for inx := range list {
		list[inx].DetailId = guid.NewGUIDString()
		list[inx].OrdersId = ordersID
		list[inx].CreateDate = time.Now()
	}

	if _, err := m.db.Insert(list); err != nil {
		utils.LogError("保存订单明细失败: " + err.Error())

		return errors.New("保存订单明细失败")
	}

	return nil
}

// SaveOrdersCoupon 保存订单优惠券
func (m *GoodsDAO) SaveOrdersCoupon(coupon *model.TaGoodsOrdersCoupon, order *model.TaGoodsOrders) error {
	if order.OrdersId == "" {
		return errors.New("内部错误, 订单事务顺序出错")
	}
	coupon.OrdersCouponId = utils.GetGUID()
	coupon.Status = models.STATUS_NORMAL
	coupon.CreateDate = time.Now().Local()
	if _, err := m.db.Insert(coupon); err != nil {
		return err
	}

	// var couponAmount float64 = 0

	// // 补充优惠券表字段
	// for inx := range coupons {
	// 	coupons[inx].OrdersCouponId = guid.NewGUIDString()
	// 	coupons[inx].OrdersId = order.OrdersId
	// 	coupons[inx].OrdersNo = order.OrdersNo
	// 	coupons[inx].OrgId = order.OrgId
	// 	coupons[inx].Status = models.STATUS_NORMAL
	// 	coupons[inx].CreateDate = time.Now().Local()

	// 	userAmount, _ := strconv.ParseFloat(coupons[inx].UsedAmount, 64)
	// 	couponAmount += userAmount
	// }

	// // 反向更新 订单主表
	// totalAmount, _ := strconv.ParseFloat(order.Amount, 64)

	// order.CouponAmount = strconv.FormatFloat(couponAmount, 'f', -1, 32)
	// order.ActualAmount = strconv.FormatFloat((totalAmount - couponAmount), 'f', -1, 32)

	// if _, err := m.db.Insert(&coupons); err != nil {
	// 	return err
	// }

	// if _, err := m.db.
	// 	Where("orders_id=?", order.OrdersId).
	// 	Cols([]string{"coupon_amount", "actual_amount"}...).
	// 	Update(order); err != nil {
	// 	return err
	// }

	return nil
}

func (m *GoodsDAO) getCaseByID(caseID string) (*model.SysCase, error) {
	cs := new(model.SysCase)
	if _, err := m.db.Where("case_id=?", caseID).Get(cs); err != nil {
		return nil, err
	}

	return cs, nil
}

// GetOrdersByCaseID 获取订单列表
func (m *GoodsDAO) GetOrdersByCaseID(caseID, userType string, makeStatus []string, orderDate []time.Time, limit []int) ([]OrdersWithGoods, int64, error) {
	var orderList []OrdersWithGoods

	query := "select a.*,b.area_icon,b.area_icon_white from ta_goods_orders a inner join sys_case_area b on a.area_id=b.area_id where a.case_id = ? and a.status = ?"

	// 制作状态
	if makeStatus != nil && len(makeStatus) > 0 {
		query += ` and a.make_status in ('` + strings.Join(makeStatus, "','") + `')`
	}

	// 用户类型
	if userType != "" {
		query += ` and a.user_type = '` + userType + `'`
	}

	// 订单起始时间
	if orderDate != nil && len(orderDate) > 0 {
		query += ` and DATE_FORMAT(a.create_date, '%Y-%m-%d %H:%i') >= '` + orderDate[0].Format("2006-01-02 15:04") + `'`

		if len(orderDate) > 1 {
			query += ` and DATE_FORMAT(a.create_date, '%Y-%m-%d %H:%i') <= '` + orderDate[1].Format("2006-01-02 15:04") + `'`
		}
	}

	query += " order by a.create_date asc"

	var cnt int64
	if limit != nil && len(limit) > 1 {
		var err error
		cnt, err = utils.NewPageNaviEngine(m.ctx).GetPageList(&orderList, query, limit, caseID, models.STATUS_NORMAL)
		if err != nil {
			return nil, 0, err
		}
	} else {
		if err := m.db.SQL(query, caseID, models.STATUS_NORMAL).Find(&orderList); err != nil {
			return nil, 0, err
		}
	}

	for i, o := range orderList {
		details, err := m.GetOrderDetailByOrder(o.OrdersId)
		if err != nil {
			return nil, 0, err
		}
		orderList[i].Goods = details
	}

	return orderList, cnt, nil
}

// GetOrderDetailByOrder 获取商品明细
func (m *GoodsDAO) GetOrderDetailByOrder(orderid string) ([]DetailWithType, error) {
	var details []DetailWithType
	sql := `select a.*,b.type_id,c.type_name from ta_goods_orders_detail a inner join ta_goods b on a.goods_id=b.goods_id inner join td_goods_type c on b.type_id=c.type_id where a.orders_id=?`
	if err := m.db.Sql(sql, orderid).Find(&details); err != nil {
		return nil, err
	}
	return details, nil
}

// GetOrdersByRecord 根据
func (m *GoodsDAO) GetOrdersByRecord(recordid string) ([]OrdersWithGoods, error) {
	var orderList []OrdersWithGoods
	sql := `select a.*, c.case_name from ta_goods_orders a inner join sys_case_record b on a.case_id=b.case_id and date_format(a.create_date,'%Y-%m')=b.month
	inner join sys_case c on a.case_id=c.case_id
	where b.record_id='` + recordid +
		`' and a.user_type != '` + USERTYPE_CUSTOMER + `' and b.status=` + strconv.Itoa(models.STATUS_NORMAL) + ` and a.status=` + strconv.Itoa(models.STATUS_NORMAL)
	err := m.db.Sql(sql).Find(&orderList)
	if err != nil {
		return nil, err
	}
	for i, o := range orderList {
		details, err := m.GetOrderDetailByOrder(o.OrdersId)
		if err != nil {
			return nil, err
		}
		orderList[i].Goods = details
	}
	return orderList, nil
}

// GetCustomerOrders 获取用户订单
func (m *GoodsDAO) GetCustomerOrders(custID string, page, pageSize int) ([]OrdersWithGoods, error) {
	var orderList []OrdersWithGoods
	sql := `select a.*, c.case_name from ta_goods_orders a 
	inner join sys_case c on a.case_id=c.case_id
	where a.user_id='` + custID +
		`' and c.status=` + strconv.Itoa(models.STATUS_NORMAL) + ` and a.status=` + strconv.Itoa(models.STATUS_NORMAL)
	sql += ` order by a.create_date desc limit ` + strconv.Itoa((page-1)*pageSize) + `, ` + strconv.Itoa(pageSize)
	err := m.db.Sql(sql).Find(&orderList)
	if err != nil {
		return nil, err
	}
	for i, o := range orderList {
		details, err := m.GetOrderDetailByOrder(o.OrdersId)
		if err != nil {
			return nil, err
		}
		orderList[i].Goods = details
	}
	return orderList, nil
}

// GetCustomerOrdersCount 获取用户订单count
func (m *GoodsDAO) GetCustomerOrdersCount(custID string) (int, error) {
	var orderList []OrdersWithGoods
	sql := `select a.*, c.case_name from ta_goods_orders a 
	inner join sys_case c on a.case_id=c.case_id
	where a.user_id='` + custID +
		`' and c.status=` + strconv.Itoa(models.STATUS_NORMAL) + ` and a.status=` + strconv.Itoa(models.STATUS_NORMAL)
	err := m.db.Sql(sql).Find(&orderList)
	if err != nil {
		return 0, err
	}
	return len(orderList), nil
}

// GetOrdersCountByCase 根据案场获取订单信息
func (m *GoodsDAO) GetOrdersCountByCase(caseid, userid string) (int64, error) {
	order := new(model.TaGoodsOrders)
	total, err := m.db.Where("case_id = ?", caseid).And("user_id=?", userid).And("status>"+strconv.Itoa(models.STATUS_DEL)).SumInt(order, "orders_num")
	return total, err
}

// GetMonthOrdersCountByCase 获取案场本月的订单数量
func (m *GoodsDAO) GetMonthOrdersCountByCase(caseid, userid string) (int64, error) {
	order := new(model.TaGoodsOrders)
	total, err := m.db.Where("case_id = ?", caseid).And("user_id=?", userid).And("status>"+strconv.Itoa(models.STATUS_DEL)).And("date_format(create_date,'%Y-%m')=date_format(NOW(),'%Y-%m')").SumInt(order, "orders_num")
	return total, err
}

// UpdateOrdersMake 更新订单制作状态
func (m *GoodsDAO) UpdateOrdersMake(orders model.TaGoodsOrders) error {
	cols := []string{
		"make_status",
	}
	_, err := m.db.Cols(cols...).Where("orders_id=?", orders.OrdersId).Update(orders)
	return err
}

// GetOrderDetailByUser 获取用户所有下单信息
func (m *GoodsDAO) GetOrderDetailByUser(caseid, userid string) ([]model.TaGoodsOrdersDetail, error) {
	var details []model.TaGoodsOrdersDetail
	sql := `select a.* from ta_goods_orders_detail a inner join ta_goods_orders b on a.orders_id=b.orders_id
		inner join ta_customer c on b.user_id = c.customer_id
	where c.user_id=? and b.status>? and b.case_id=?`
	if err := m.db.Sql(sql, userid, models.STATUS_DEL, caseid).Find(&details); err != nil {
		return nil, err
	}
	return details, nil
}
