package statistics

import (
	"errors"
	"strconv"
	"strings"
	"time"
)

// CardCouponStatistics 商品统计信息
type CardCouponStatistics struct {
	CaseName   string
	CName      string
	TypeName   string
	SendType   string
	Price      string
	SentCount  int
	UsedCount  int
	CreateDate time.Time
}

// CardCouponStatistics 获取商品统计信息
func (m *StatisticsDAO) CardCouponStatistics(caseids, ctype, name string, page, pageSize int) ([]CardCouponStatistics, error) {
	var cardCoupons []CardCouponStatistics
	couponsql := `SELECT
		b.case_name,
		b.case_id,
		a.coupon_name AS c_name,
		(
			CASE
			WHEN coupon_type = 'course' THEN
				'课程优惠券'
			ELSE
				'商品优惠券'
			END
		) type_name,
		a.send_type,
		a.price,
		a.sent_count,
		a.used_count,
		a.create_date
	FROM
		ta_coupon a
	INNER JOIN sys_case b ON a.case_id = b.case_id
	where b.case_id in ('` + strings.Replace(caseids, ",", "','", -1) + `')`

	cardsql := `SELECT
		b.case_name,
		b.case_id,
		a.card_name AS c_name,
		'课程体验卡' AS type_name,
		a.send_type,
		a.price,
		a.sent_count,
		a.used_count,
		a.create_date
	FROM
		ta_coupon_card a
	INNER JOIN sys_case b ON a.case_id = b.case_id
	where b.case_id in ('` + strings.Replace(caseids, ",", "','", -1) + `')`

	// gymcardsql := `SELECT
	// 	b.case_name,
	// 	b.case_id,
	// 	a.gym_card_name,
	// 	'健身卡' AS type_name,
	// 	a.send_type,
	// 	a.price,
	// 	a.sent_count,
	// 	a.used_count,
	// 	a.create_date
	// FROM
	// 	ta_gym_card a
	// INNER JOIN sys_case b ON a.case_id = b.case_id
	// where b.case_id in ('` + strings.Replace(caseids, ",", "','", -1) + `')`

	sql := ""
	if ctype != "" {
		switch ctype {
		case "card":
			sql = "select * from (" + cardsql + ") tab"
			break
		case "goodscoupon":
			sql = "select * from (" + couponsql + " and a.coupon_type='goods') tab"
			break
		case "coursecoupon":
			sql = "select * from (" + couponsql + " and a.coupon_type='course') tab"
			break
		// case "gym":
		// 	sql = "select * from (" + gymcardsql + ") tab"
		// 	break
		default:
			return nil, errors.New("类型不正确")
		}
	} else {
		sql = "select * from (" + cardsql + " union " + couponsql + ") tab"
	}

	if name != "" {
		sql = sql + ` where c_name like '%` + name + `%'`
	}
	sql = sql + ` order by create_date desc limit ` + strconv.Itoa((page-1)*pageSize) + `, ` + strconv.Itoa(pageSize)
	err := m.db.Sql(sql).Find(&cardCoupons)
	return cardCoupons, err
}

// CardCouponStatisticsCount 获取总数
func (m *StatisticsDAO) CardCouponStatisticsCount(caseids, ctype, name string) ([]CardCouponStatistics, error) {
	var cardCoupons []CardCouponStatistics
	couponsql := `SELECT
		b.case_name,
		b.case_id,
		a.coupon_name AS c_name,
		(
			CASE
			WHEN coupon_type = 'course' THEN
				'课程优惠券'
			ELSE
				'商品优惠券'
			END
		) type_name,
		a.send_type,
		a.price,
		a.sent_count,
		a.used_count,
		a.create_date
	FROM
		ta_coupon a
	INNER JOIN sys_case b ON a.case_id = b.case_id
	where b.case_id in ('` + strings.Replace(caseids, ",", "','", -1) + `')`

	cardsql := `SELECT
		b.case_name,
		b.case_id,
		a.card_name AS c_name,
		'课程体验卡' AS type_name,
		a.send_type,
		a.price,
		a.sent_count,
		a.used_count,
		a.create_date
	FROM
		ta_coupon_card a
	INNER JOIN sys_case b ON a.case_id = b.case_id
	where b.case_id in ('` + strings.Replace(caseids, ",", "','", -1) + `')`

	// gymcardsql := `SELECT
	// 	b.case_name,
	// 	b.case_id,
	// 	a.gym_card_name,
	// 	'健身卡' AS type_name,
	// 	a.send_type,
	// 	a.price,
	// 	a.sent_count,
	// 	a.used_count,
	// 	a.create_date
	// FROM
	// 	ta_gym_card a
	// INNER JOIN sys_case b ON a.case_id = b.case_id
	// where b.case_id in ('` + strings.Replace(caseids, ",", "','", -1) + `')`

	sql := ""
	if ctype != "" {
		switch ctype {
		case "card":
			sql = "select * from (" + cardsql + ") tab"
			break
		case "goodscoupon":
			sql = "select * from (" + couponsql + " and a.coupon_type='goods') tab"
			break
		case "coursecoupon":
			sql = "select * from (" + couponsql + " and a.coupon_type='course') tab"
			break
		// case "gym":
		// 	sql = "select * from (" + gymcardsql + ") tab"
		// 	break
		default:
			return cardCoupons, errors.New("类型不正确")
		}
	} else {
		sql = "select * from (" + cardsql + " union " + couponsql + ") tab"
	}

	if name != "" {
		sql = sql + ` where c_name like '%` + name + `%'`
	}
	err := m.db.Sql(sql).Find(&cardCoupons)
	return cardCoupons, err
}

// CardCouponUsedStatistics 卡券使用情况
type CardCouponUsedStatistics struct {
	CaseId         string
	CaseName       string
	CardCouponName string
	TargetName     string
	ReceivingType  string
	Name           string
	CustomerName   string
	Phone          string
	SalesId        string
	SalesName      string
	StartDate      time.Time
	EndDate        time.Time
	ReceiveDate    time.Time
	UsedDate       time.Time
	VerifyStatus   string
}

// CardCouponUsedStatistics 获取卡券使用情况统计
func (m *StatisticsDAO) CardCouponUsedStatistics(caseids, tel, name, receivetype, begindate, enddate, status string, page, pageSize int) ([]CardCouponUsedStatistics, error) {
	var cardCoupons []CardCouponUsedStatistics
	cardsql := `SELECT
							d.case_name,
							d.case_id,
							a.customer_card_name as card_coupon_name,
							b.target_name,
							a.receiving_type,
							c.name,
							c.customer_name,
							c.phone,
							a.sales_id,
							a.sales_name,
							a.start_date,
							a.end_date,
							a.receive_date,
							a.receive_date as used_date,
							'used' as verify_status
						FROM
							ta_customer_card a
						INNER JOIN ta_coupon_card_target b ON b.card_id = a.card_id
						INNER JOIN ta_customer c ON a.customer_id = c.customer_id
						INNER JOIN sys_case d ON a.case_id = d.case_id
						WHERE d.case_id in ('` + strings.Replace(caseids, ",", "','", -1) + `')`

	couponsql := `SELECT
									b.case_name,
									b.case_id,
									a.customer_coupon_name as card_coupon_name,
									(
										CASE
										WHEN cp.coupon_type = 'course' THEN
											d.course_name
										ELSE
											c.coupon_name
										END
									) as target_name,
									a.receiving_type,
									cst.name,
									cst.customer_name,
									cst.phone,
									a.sales_id,
									a.sales_name,
									a.start_date,
									a.end_date,
									a.receive_date,
									(
										CASE
										WHEN cp.coupon_type = 'course' THEN
											d.create_date
										ELSE
											c.create_date
										END
									) as used_date,
									(
										CASE
										WHEN NOW() > a.end_date THEN
											'expire'
										WHEN use_date IS NULL THEN
											'useable'
										ELSE
											'used'
										END
									) AS verify_status
								FROM
									ta_customer_coupon a
								INNER JOIN sys_case b ON a.case_id = b.case_id
								INNER JOIN ta_customer cst ON a.customer_id = cst.customer_id
								INNER JOIN ta_coupon cp ON a.coupon_id = cp.coupon_id
								LEFT JOIN ta_goods_orders_coupon c ON a.coupon_id = c.coupon_id
								LEFT JOIN ta_course_orders_coupon d ON a.coupon_id = d.coupon_id
								WHERE b.case_id in ('` + strings.Replace(caseids, ",", "','", -1) + `')`

	sql := "select * from (" + cardsql + " union " + couponsql + ") tab where (1=1)"

	if name != "" {
		sql = sql + ` and tab.card_coupon_name like '%` + name + `%'`
	}
	if tel != "" {
		sql = sql + ` and tab.phone like '%` + tel + `%'`
	}
	if receivetype != "" {
		sql = sql + ` and tab.receiving_type = '` + receivetype + `'`
	}
	if status != "" {
		sql = sql + ` and tab.verify_status = '` + receivetype + `'`
	}
	if begindate != "" {
		sql = sql + ` and DATE_FORMAT(tab.receive_date, '%Y-%m-%d') >= '` + begindate + `'`
	}
	if enddate != "" {
		sql = sql + ` and DATE_FORMAT(tab.receive_date, '%Y-%m-%d') <= '` + enddate + `'`
	}
	sql = sql + ` order by receive_date desc limit ` + strconv.Itoa((page-1)*pageSize) + `, ` + strconv.Itoa(pageSize)

	err := m.db.Sql(sql).Find(&cardCoupons)
	return cardCoupons, err
}

// CardCouponUsedCountStatistics 获取卡券使用情况统计
func (m *StatisticsDAO) CardCouponUsedCountStatistics(caseids, tel, name, receivetype, begindate, enddate, status string) ([]CardCouponUsedStatistics, error) {
	var cardCoupons []CardCouponUsedStatistics
	cardsql := `SELECT
							d.case_name,
							d.case_id,
							a.customer_card_name as card_coupon_name,
							b.target_name,
							a.receiving_type,
							c.name,
							c.customer_name,
							c.phone,
							a.sales_id,
							a.sales_name,
							a.start_date,
							a.end_date,
							a.receive_date,
							a.receive_date as used_date,
							a.verify_status
						FROM
							ta_customer_card a
						INNER JOIN ta_coupon_card_target b ON b.card_id = a.card_id
						INNER JOIN ta_customer c ON a.customer_id = c.customer_id
						INNER JOIN sys_case d ON a.case_id = d.case_id
						WHERE d.case_id in ('` + strings.Replace(caseids, ",", "','", -1) + `')`

	couponsql := `SELECT
									b.case_name,
									b.case_id,
									a.customer_coupon_name as card_coupon_name,
									(
										CASE
										WHEN cp.coupon_type = 'course' THEN
											d.course_name
										ELSE
											c.coupon_name
										END
									) as target_name,
									a.receiving_type,
									cst.name,
									cst.customer_name,
									cst.phone,
									a.sales_id,
									a.sales_name,
									a.start_date,
									a.end_date,
									a.receive_date,
									(
										CASE
										WHEN cp.coupon_type = 'course' THEN
											d.create_date
										ELSE
											c.create_date
										END
									) as used_date,
									(
										CASE
										WHEN NOW() > a.end_date THEN
											'expire'
										WHEN use_date IS NULL THEN
											'useable'
										ELSE
											'used'
										END
									) AS verify_status
								FROM
									ta_customer_coupon a
								INNER JOIN sys_case b ON a.case_id = b.case_id
								INNER JOIN ta_customer cst ON a.customer_id = cst.customer_id
								INNER JOIN ta_coupon cp ON a.coupon_id = cp.coupon_id
								LEFT JOIN ta_goods_orders_coupon c ON a.coupon_id = c.coupon_id
								LEFT JOIN ta_course_orders_coupon d ON a.coupon_id = d.coupon_id
								WHERE b.case_id in ('` + strings.Replace(caseids, ",", "','", -1) + `')`

	sql := "select * from (" + cardsql + " union " + couponsql + ") tab where (1=1)"

	if name != "" {
		sql = sql + ` and tab.card_coupon_name like '%` + name + `%'`
	}
	if tel != "" {
		sql = sql + ` and tab.phone like '%` + tel + `%'`
	}
	if receivetype != "" {
		sql = sql + ` and tab.receiving_type = '` + receivetype + `'`
	}
	if status != "" {
		sql = sql + ` and tab.verify_status = '` + receivetype + `'`
	}
	if begindate != "" {
		sql = sql + ` and DATE_FORMAT(tab.receive_date, '%Y-%m-%d') >= '` + begindate + `'`
	}
	if enddate != "" {
		sql = sql + ` and DATE_FORMAT(tab.receive_date, '%Y-%m-%d') <= '` + enddate + `'`
	}
	err := m.db.Sql(sql).Find(&cardCoupons)
	return cardCoupons, err
}

// CardCouponVerifyStatistics 卡券使用情况
type CardCouponVerifyStatistics struct {
	CaseId       string
	CaseName     string
	CourseName   string
	BeginDate    time.Time
	EndDate      time.Time
	Name         string
	CustomerName string
	Phone        string
	CreateDate   time.Time
	VerifyDate   time.Time
	Status       string
}

// CardCouponVerifyStatistics 获取卡券核销情况统计
func (m *StatisticsDAO) CardCouponVerifyStatistics(caseids, tel, name, status string, page, pageSize int) ([]CardCouponVerifyStatistics, error) {
	var cardCoupons []CardCouponVerifyStatistics
	sql := `SELECT
							c.case_id,
							c.case_name,
							e.course_name,
							e.begin_date,
							e.end_date,
							d.name,
							d.customer_name,
							d.phone,
							d.recommend_name,
							a.create_date,
							b.verify_date,
							(
								CASE
								WHEN b.verify_date IS NULL THEN
									'useable'
								ELSE
									f.verify_status
								END
							) AS verify_status
						FROM
							ta_customer_course a
						INNER JOIN (
							SELECT
								customer_course_id,
								min(verify_date) AS verify_date
							FROM
								ta_customer_course_detail
							GROUP BY
								customer_course_id
						) b ON a.customer_course_id = b.customer_course_id
						INNER JOIN sys_case c ON a.case_id = c.case_id
						INNER JOIN ta_customer d ON a.customer_id = d.customer_id
						INNER JOIN ta_course e ON a.course_id = e.course_id
						LEFT JOIN ta_customer_course_detail f ON b.customer_course_id = f.customer_course_id
						AND b.verify_date = f.verify_date
						WHERE c.case_id in ('` + strings.Replace(caseids, ",", "','", -1) + `')`

	if name != "" {
		sql = sql + ` and e.course_name like '%` + name + `%'`
	}
	if tel != "" {
		sql = sql + ` and d.phone like '%` + tel + `%'`
	}
	if status != "" {
		if status == "useable" {
			sql = sql + ` and b.verify_date is null`
		} else {
			sql = sql + ` and f.verify_status ='` + status + `'`
		}
	}
	sql = sql + ` order by a.create_date desc limit ` + strconv.Itoa((page-1)*pageSize) + `, ` + strconv.Itoa(pageSize)

	err := m.db.Sql(sql).Find(&cardCoupons)
	return cardCoupons, err
}

// CardCouponVerifyStatisticsCount 获取卡券核销情况数量统计
func (m *StatisticsDAO) CardCouponVerifyStatisticsCount(caseids, tel, name, status string) ([]CardCouponVerifyStatistics, error) {
	var cardCoupons []CardCouponVerifyStatistics
	sql := `SELECT
							c.case_id,
							c.case_name,
							e.course_name,
							e.begin_date,
							e.end_date,
							d.name,
							d.customer_name,
							d.phone,
							d.recommend_name,
							a.create_date,
							b.verify_date,
							(
								CASE
								WHEN b.verify_date IS NULL THEN
									'useable'
								ELSE
									f.verify_status
								END
							) AS verify_status
						FROM
							ta_customer_course a
						INNER JOIN (
							SELECT
								customer_course_id,
								min(verify_date) AS verify_date
							FROM
								ta_customer_course_detail
							GROUP BY
								customer_course_id
						) b ON a.customer_course_id = b.customer_course_id
						INNER JOIN sys_case c ON a.case_id = c.case_id
						INNER JOIN ta_customer d ON a.customer_id = d.customer_id
						INNER JOIN ta_course e ON a.course_id = e.course_id
						LEFT JOIN ta_customer_course_detail f ON b.customer_course_id = f.customer_course_id
						AND b.verify_date = f.verify_date
						WHERE c.case_id in ('` + strings.Replace(caseids, ",", "','", -1) + `')`

	if name != "" {
		sql = sql + ` and e.course_name like '%` + name + `%'`
	}
	if tel != "" {
		sql = sql + ` and d.phone like '%` + tel + `%'`
	}
	if status != "" {
		if status == "useable" {
			sql = sql + ` and b.verify_date is null`
		} else {
			sql = sql + ` and f.verify_status ='` + status + `'`
		}
	}
	err := m.db.Sql(sql).Find(&cardCoupons)
	return cardCoupons, err
}
