package statistics

import (
	"spaceofcheng/services/models/model"
	"strconv"
	"strings"
	"time"
)

// GoodsStatistics 商品统计信息
type GoodsStatistics struct {
	CaseName    string
	GoodsName   string
	TypeName    string
	Price       string
	Total       int
	FinishTotal int
	Amount      string
}

// GetGoodsStatistics 获取商品统计信息
func (m *StatisticsDAO) GetGoodsStatistics(caseids, typeid, name string, page, pageSize int) ([]GoodsStatistics, error) {
	var goods []GoodsStatistics
	sql := `SELECT
					e.case_name,
					b.goods_name,
					c.type_name,
					b.price,
					sum(a.number) AS total,
					sum(
						(
							CASE
							WHEN d.make_status = 'finished' THEN
								1
							ELSE
								0
							END
						)
					) AS finishtotal,
					b.price * sum(a.number) AS amount
				FROM
					ta_goods_orders_detail a
				INNER JOIN ta_goods b ON a.goods_id = b.goods_id
				INNER JOIN td_goods_type c ON b.type_id = c.type_id
				INNER JOIN ta_goods_orders d ON a.orders_id = d.orders_id
				inner join sys_case e on b.case_id=e.case_id 
				where e.case_id in ('` + strings.Replace(caseids, ",", "','", -1) + `')`

	if typeid != "" {
		sql = sql + ` and c.type_id='` + typeid + `'`
	}

	if name != "" {
		sql = sql + ` and b.goods_name like '%` + name + `%'`
	}
	sql = sql + ` GROUP BY
					e.case_name,
					b.goods_name,
					c.type_name,
					b.price limit ` + strconv.Itoa((page-1)*pageSize) + `, ` + strconv.Itoa(pageSize)
	err := m.db.Sql(sql).Find(&goods)
	return goods, err
}

// GetGoodsStatisticsCount 获取总数
func (m *StatisticsDAO) GetGoodsStatisticsCount(caseids, typeid, name string) ([]GoodsStatistics, error) {
	var goods []GoodsStatistics
	sql := `SELECT
					e.case_name,
					b.goods_name,
					c.type_name,
					b.price,
					sum(a.number) AS total,
					sum(
						(
							CASE
							WHEN d.make_status = 'finished' THEN
								1
							ELSE
								0
							END
						)
					) AS finishtotal,
					b.price * sum(a.number) AS amount
				FROM
					ta_goods_orders_detail a
				INNER JOIN ta_goods b ON a.goods_id = b.goods_id
				INNER JOIN td_goods_type c ON b.type_id = c.type_id
				INNER JOIN ta_goods_orders d ON a.orders_id = d.orders_id
				inner join sys_case e on b.case_id=e.case_id 
				where e.case_id in ('` + strings.Replace(caseids, ",", "','", -1) + `')`

	if typeid != "" {
		sql = sql + ` and c.type_id='` + typeid + `'`
	}

	if name != "" {
		sql = sql + ` and b.goods_name like '%` + name + `%'`
	}
	sql = sql + ` GROUP BY
					e.case_name,
					b.goods_name,
					c.type_name,
					b.price`
	err := m.db.Sql(sql).Find(&goods)
	return goods, err
}

// GoodsOrdersStatistics 商品订单统计信息
type GoodsOrdersStatistics struct {
	OrdersId     string
	OrdersNo     string
	CaseName     string
	UserType     string
	CreateDate   time.Time
	Status       int
	MakeStatus   string
	AreaName     string
	TableNo      string
	CustomerName string
	Phone        string
	Name         string
	Remark       string
	Details      []model.TaGoodsOrdersDetail
}

// GetGoodsOrderStatistics 获取商品订单
func (m *StatisticsDAO) GetGoodsOrderStatistics(status, usertype, caseids, begindate, enddate string, page, pageSize int) ([]GoodsOrdersStatistics, error) {
	var orders []GoodsOrdersStatistics
	sql := `SELECT
						a.orders_id,
						a.orders_no,
						d.case_name,
						a.user_type,
						a.create_date,
						a.status,
						a.make_status,
						a.area_name,
						a.table_no,
						c.customer_name,
						c. NAME,
						c.phone,
						a.remark
					FROM
						ta_goods_orders a
					INNER JOIN ta_customer b ON a.user_id = b.customer_id
					INNER JOIN ta_customer c ON b.customer_id = c.customer_id
					INNER JOIN sys_case d ON a.case_id = d.case_id
					where d.case_id in ('` + strings.Replace(caseids, ",", "','", -1) + `')`

	if usertype != "" {
		sql = sql + ` and a.user_type='` + usertype + `'`
	}
	if begindate != "" {
		sql = sql + ` and DATE_FORMAT(a.create_date, '%Y-%m-%d') >= '` + begindate + `'`
	}
	if enddate != "" {
		sql = sql + ` and DATE_FORMAT(a.create_date, '%Y-%m-%d') <= '` + enddate + `'`
	}
	if status != "" {
		sql = sql + ` and a.make_status='` + status + `'`
	}
	sql = sql + ` order by a.create_date desc limit ` + strconv.Itoa((page-1)*pageSize) + `, ` + strconv.Itoa(pageSize)

	err := m.db.Sql(sql).Find(&orders)
	if err != nil {
		return nil, err
	}
	for inx, item := range orders {
		var details []model.TaGoodsOrdersDetail
		err := m.db.Where("orders_id=?", item.OrdersId).Find(&details)
		if err != nil {
			return nil, err
		}
		orders[inx].Details = details
	}
	return orders, err
}

// GetGoodsOrderStatisticsCount 获取商品订单Count
func (m *StatisticsDAO) GetGoodsOrderStatisticsCount(status, usertype, caseids, begindate, enddate string) ([]GoodsOrdersStatistics, error) {
	var orders []GoodsOrdersStatistics
	sql := `SELECT
						a.orders_id,
						a.orders_no,
						d.case_name,
						a.user_type,
						a.create_date,
						a.status,
						a.make_status,
						a.area_name,
						a.table_no,
						c.customer_name,
						c. NAME,
						c.phone,
						a.remark
					FROM
						ta_goods_orders a
					INNER JOIN ta_customer b ON a.user_id = b.customer_id
					INNER JOIN ta_customer c ON b.customer_id = c.customer_id
					INNER JOIN sys_case d ON a.case_id = d.case_id
					where d.case_id in ('` + strings.Replace(caseids, ",", "','", -1) + `')`

	if usertype != "" {
		sql = sql + ` and a.user_type='` + usertype + `'`
	}
	if begindate != "" {
		sql = sql + ` and DATE_FORMAT(a.create_date, '%Y-%m-%d') >= '` + begindate + `'`
	}
	if enddate != "" {
		sql = sql + ` and DATE_FORMAT(a.create_date, '%Y-%m-%d') <= '` + enddate + `'`
	}
	if status != "" {
		sql = sql + ` and a.make_status='` + status + `'`
	}

	err := m.db.Sql(sql).Find(&orders)
	return orders, err
}
