package cases

import (
	"spaceofcheng/services/models/cases"
	"spaceofcheng/services/models/marketing"
	"spaceofcheng/services/models/model"
	"spaceofcheng/services/utils"
	"time"

	"github.com/astaxie/beego"
)

// SigninServ 系统处理
type SigninServ struct {
	ctx     *utils.Context
	dao     *cases.SigninDAO
	marDao  *marketing.MarketingDAO
	caseDao *cases.CaseDAO
}

// NewSigninServ 初始化
func NewSigninServ(ctx *utils.Context) *SigninServ {
	return &SigninServ{
		ctx:     ctx,
		dao:     cases.NewSigninDAO(ctx),
		marDao:  marketing.NewMarketingDAO(ctx),
		caseDao: cases.NewCaseDAO(ctx),
	}
}

//GetSigninWhere
/**
GetSigninWhere 根据条件查询签到记录
		 条件:
				 selectType: 今天,本周,本月,今年,全部
				 caseid: 案场ID
*/
func (s *SigninServ) GetSigninWhere(selectType int, caseids string, page, pageSize int) (map[string]interface{}, error) {

	info, err := s.dao.GetSigninWhere(selectType, caseids, page, pageSize)
	var total int
	if err != nil {
		beego.Error(err)
		return nil, err
	}
	if len(info) > 0 {
		total = len(info)
	}

	return map[string]interface{}{
		"list":     info,
		"pagesize": pageSize,
		"pagenum":  total,
		"page":     page,
	}, err
}

// 添加 签到记录
func (s *SigninServ) AddSignin(signin model.TaCheckinRecord, caseId string, activityId string) (*model.TaCheckinRecord, error) {
	if signin.Phone == "" {
		return nil, utils.LogError("没有Phone手机号")
	}
	if caseId == "" {
		return nil, utils.LogError("没有案场ID")
	}

	if signin.OrgId == "" {
		return nil, utils.LogError("没有机构ID")
	}

	// 获取上下文的用户
	user := s.ctx.Get("customer").(model.TaCustomer)
	userMap := s.ctx.Get("userMap").(model.TaUserMapping)
	casesInfo, err := s.caseDao.GetCaseByID(caseId)
	// activityInfo, err := s.marDao.GetMarketingById(activityId)

	// openId
	signin.OpenId = userMap.Openid
	// 微信昵称
	signin.CustomerName = user.CustomerName
	// 客户姓名
	signin.Name = user.Name
	// 客户ID
	signin.CustomerId = user.CustomerId

	// 案场
	signin.CaseId = caseId
	signin.CaseName = casesInfo.CaseName

	exists, _ := s.dao.ValidCheckIn(caseId, userMap.Openid, time.Now().Local().Format("2006-01-02"))
	if exists != nil && exists.OpenId != "" {
		return exists, nil
	}

	// 活动
	// signin.ActivityId = activityInfo.ActivityId
	// signin.ActivityName = activityInfo.ActivityName

	info, err := s.dao.AddSignin(signin)
	if err != nil {
		return nil, err
	}

	return &info, err
}

// GetSigninExcel 根据条件查询出签到记录
func (s *SigninServ) GetSigninExcel(selectType int, caseids string) ([]cases.SigninInfo, error) {

	info, err := s.dao.GetSigninExcel(selectType, caseids)
	if err != nil {
		beego.Error(err)
		return nil, err
	}

	return info, err
}
