package events

import (
	"fmt"
	"spaceofcheng/services/models"
	"spaceofcheng/services/models/model"
	"spaceofcheng/services/utils"
	"sync"

	"github.com/zjxpcyc/tinyevent"
)

var mtx sync.Mutex

// EvtActions 内置 Action 列表
var EvtActions = map[string]tinyevent.Action{
	// 测试
	ActTest: tsAction,

	// 赠送卡券
	ActGiveCoupon: giveCoupon,
}

// ListenAllEvent 监听所有事件
func ListenAllEvent(orgID, caseID string) {
	mtx.Lock()
	defer mtx.Unlock()

	evtEngID := utils.EngineIDBy(orgID, caseID)
	evtEngine := utils.ResetEventEngineBus(evtEngID)

	// 所有需要执行的事件
	allEvts, err := getAllEvents(orgID, caseID)
	if err != nil {
		utils.LogError(err)
	}

	// 刷新中控中心
	for evt, acts := range allEvts {
		for _, actKey := range acts {
			act, ok := EvtActions[actKey]
			if ok {
				evtEngine.ListenEvent(evt, act)
			}
		}
	}
}

// getAllEvents 获取数据库配置
func getAllEvents(orgID, caseID string) (map[string][]string, error) {
	db := models.DBEngine

	var evts []model.SysActivity
	var acts []model.SysActivityAction

	if err := db.Where("org_id=?", orgID).
		And("case_id=?", caseID).
		And("status=?", models.STATUS_NORMAL).
		Find(&evts); err != nil {
		utils.LogError("查询营销活动配置表失败: " + err.Error())
		return nil, err
	}

	res := make(map[string][]string)
	for _, evtSetting := range evts {
		res[evtSetting.ActivityType] = make([]string, 0)
		acts = make([]model.SysActivityAction, 0)

		if err := db.Where("activity_id=?", evtSetting.ActivityId).Find(&acts); err != nil {
			utils.LogError("查询营销活动配置明细失败: " + err.Error())
			return nil, err
		}

		for _, act := range acts {
			res[evtSetting.ActivityType] = append(res[evtSetting.ActivityType], act.ActiveType)
		}
	}

	return res, nil
}

var tsAction = func(e tinyevent.Event) error {
	str := e.Payload.(string)
	fmt.Println(str)
	return nil
}
