package events

import (
	"errors"
	"spaceofcheng/services/models"
	"spaceofcheng/services/models/model"
	"spaceofcheng/services/service/coupon"
	"spaceofcheng/services/utils"

	"github.com/zjxpcyc/tinyevent"
)

// giveCoupon 赠送卡券
// Event.Payload.customer  = model.TaCustomer
var giveCoupon = func(e tinyevent.Event) error {
	hasDBError := false
	ctx := NewContext()
	defer DestroyContext(ctx, hasDBError)

	if e.Payload == nil {
		hasDBError = true
		utils.LogError("注册送券失败, 没有用户信息")
		return errors.New("注册送券失败, 没有用户信息")
	}
	dtRaw := e.Payload.(map[string]interface{})

	cust := dtRaw["customer"].(model.TaCustomer)
	caseID := cust.RecommendCase
	orgID := cust.OrgId

	query := `
		SELECT
			t.*
		FROM
			sys_activity_action t
		JOIN sys_activity s ON t.activity_id = s.activity_id
		WHERE
			s.org_id = ?
		AND s.case_id = ?
		AND s.status = ?
		AND t.active_type = ?
	`

	var gifts []model.SysActivityAction
	if err := models.DBEngine.SQL(query, orgID, caseID, models.STATUS_NORMAL, e.Name).Find(&gifts); err != nil {
		hasDBError = true
		utils.LogError("检查营销活动失败: " + err.Error())
		return errors.New("检查营销活动失败")
	}

	fromUser := model.SysUser{
		UserId:   "SYSTEM",
		UserName: "system",
	}

	cpServ := coupon.NewCouponServ(ctx)

	for _, gift := range gifts {
		switch gift.ResourceType {
		case models.GIVE_GIFT_COUPON:
			cpID := gift.ResourceDesc
			cp, err := cpServ.GetCouponByID(cpID)
			if err != nil {
				utils.LogError("获取优惠券失败: " + err.Error())
				hasDBError = true
				return errors.New("校验优惠券失败")
			}

			if err := cpServ.GiveCouponTo(&fromUser, &cust, cp, models.RECEIVEING_TYPE_EVENT); err != nil {
				hasDBError = true
				return err
			}
		case models.GIVE_GIFT_EXPERIENCECARD:
		case models.GIVE_GIFT_GYM:
		default:
		}
	}

	return nil
}
