package message

import (
	"errors"
	"spaceofcheng/services/models/course"
	"spaceofcheng/services/models/message"
	"spaceofcheng/services/models/model"
	"spaceofcheng/services/service"
	"spaceofcheng/services/utils"
)

// MessageServ 系统处理
type MessageServ struct {
	ctx       *utils.Context
	dao       *message.MessageDAO
	coursedao *course.CourseDAO
}

// NewMessageServ 初始化
func NewMessageServ(ctx *utils.Context) *MessageServ {
	return &MessageServ{
		ctx:       ctx,
		dao:       message.NewMessageDAO(ctx),
		coursedao: course.NewCourseDAO(ctx),
	}
}

// GetCmsInfoList 获取消息管理列表
func (s *MessageServ) GetCmsInfoList(title, orgid string, page int, pageSize int) (map[string]interface{}, error) {
	if pageSize == 0 {
		pageSize = service.PAGENUM
	}
	if page == 0 {
		page = 1
	}
	infos, err := s.dao.GetCmsInfoList(title, orgid, page, pageSize)
	if err != nil {
		utils.LogError("获取列表失败: " + err.Error())
		return nil, errors.New("获取列表失败")
	}
	total, err := s.dao.GetCmsInfoListCount(title, orgid)
	if err != nil {
		utils.LogError("获取列表失败: " + err.Error())
		return nil, errors.New("获取列表失败")
	}

	return map[string]interface{}{
		"list":     infos,
		"pagesize": pageSize,
		"pagenum":  total,
		"page":     page,
	}, err
}

// GetCmsInfoByLocation 根据位置ID获取消息列表
func (s *MessageServ) GetCmsInfoByLocation(locationid, orgid string) ([]model.TaCmsInfo, error) {
	infos, err := s.dao.GetCmsInfoByLocation(locationid, orgid)
	if err != nil {
		utils.LogError("获取列表失败: " + err.Error())
		return nil, errors.New("获取列表失败")
	}
	return infos, err
}

// GetCmsInfoByID 获取单条消息信息
func (s *MessageServ) GetCmsInfoByID(id string) (*model.TaCmsInfo, error) {
	info, err := s.dao.GetCmsInfoByID(id)
	if err != nil {
		utils.LogError("获取信息失败: " + err.Error())
		return nil, errors.New("获取信息失败")
	}
	return info, err
}

// SaveCmsInfo 保存消息信息
func (s *MessageServ) SaveCmsInfo(cmsinfo model.TaCmsInfo) (*model.TaCmsInfo, error) {
	if cmsinfo.InfoId == "" {
		org := s.ctx.Get("org").(model.SysOrg)
		cmsinfo.OrgId = org.OrgId
		newcmsinfo, err := s.dao.AddCmsInfo(cmsinfo)
		if err != nil {
			utils.LogError("保存信息失败: " + err.Error())
			return nil, errors.New("保存信息失败")
		}
		return newcmsinfo, nil
	}
	err := s.dao.UpdateCmsInfo(cmsinfo)
	if err != nil {
		utils.LogError("保存信息失败: " + err.Error())
		return nil, errors.New("保存信息失败")
	}
	return &cmsinfo, nil
}

// DelCmsInfo 删除消息信息
func (s *MessageServ) DelCmsInfo(infoid string) error {
	err := s.dao.DelCmsInfo(infoid)
	if err != nil {
		utils.LogError("刪除信息失败: " + err.Error())
		return errors.New("刪除信息失败")
	}
	return err
}
