package statistics

import (
	"errors"
	"spaceofcheng/services/models/statistics"
	"spaceofcheng/services/service"
	"spaceofcheng/services/utils"
)

// GetGoodsStatistics 获取商品统计信息
func (s *StatisticsServ) GetGoodsStatistics(caseids, typeid, name string, page, pageSize int) (map[string]interface{}, error) {
	if pageSize == 0 {
		pageSize = service.PAGENUM
	}
	if page == 0 {
		page = 1
	}

	if caseids == "" {
		return nil, errors.New("请先选择案场信息")
	}

	list, err := s.dao.GetGoodsStatistics(caseids, typeid, name, page, pageSize)
	if err != nil {
		utils.LogError("获取商品统计数据失败: " + err.Error())
		return nil, errors.New("获取商品统计数据失败")
	}
	all, err := s.dao.GetGoodsStatisticsCount(caseids, typeid, name)
	if err != nil {
		utils.LogError("获取商品统计数据失败: " + err.Error())
		return nil, errors.New("获取商品统计数据失败")
	}

	return map[string]interface{}{
		"list":     list,
		"pagesize": pageSize,
		"pagenum":  len(all),
		"page":     page,
	}, nil
}

// GetGoodsStatisticsExcel 获取商品统计信息导出
func (s *StatisticsServ) GetGoodsStatisticsExcel(caseids, typeid, name string) ([]statistics.GoodsStatistics, error) {
	all, err := s.dao.GetGoodsStatisticsCount(caseids, typeid, name)
	if err != nil {
		utils.LogError("获取商品统计数据失败: " + err.Error())
		return nil, errors.New("获取商品统计数据失败")
	}
	return all, nil
}

// GetGoodsOrdersStatistics 获取商品订单统计信息
func (s *StatisticsServ) GetGoodsOrdersStatistics(status, usertype, caseids, begindate, enddate string, page, pageSize int) (map[string]interface{}, error) {
	if pageSize == 0 {
		pageSize = service.PAGENUM
	}
	if page == 0 {
		page = 1
	}

	if caseids == "" {
		return nil, errors.New("请先选择案场信息")
	}

	list, err := s.dao.GetGoodsOrderStatistics(status, usertype, caseids, begindate, enddate, page, pageSize)
	if err != nil {
		utils.LogError("获取商品订单统计数据失败: " + err.Error())
		return nil, errors.New("获取商品订单统计数据失败")
	}
	all, err := s.dao.GetGoodsOrderStatisticsCount(status, usertype, caseids, begindate, enddate)
	if err != nil {
		utils.LogError("获取商品订单统计数据失败: " + err.Error())
		return nil, errors.New("获取商品订单统计数据失败")
	}

	return map[string]interface{}{
		"list":     list,
		"pagesize": pageSize,
		"pagenum":  len(all),
		"page":     page,
	}, nil
}

// GetGoodsOrdersStatisticsExcel 商品订单导出
func (s *StatisticsServ) GetGoodsOrdersStatisticsExcel(status, usertype, caseids, begindate, enddate string) ([]statistics.GoodsOrdersStatistics, error) {
	if caseids == "" {
		return nil, errors.New("请先选择案场信息")
	}

	all, err := s.dao.GetGoodsOrderStatisticsCount(status, usertype, caseids, begindate, enddate)
	if err != nil {
		utils.LogError("获取商品订单统计数据失败: " + err.Error())
		return nil, errors.New("获取商品订单统计数据失败")
	}

	return all, nil
}
