package verify

import (
	"errors"
	"spaceofcheng/services/models"
	"spaceofcheng/services/models/customer"
	"spaceofcheng/services/models/model"
	"spaceofcheng/services/models/verify"
	"spaceofcheng/services/service"
	"spaceofcheng/services/utils"
	"time"
)

// CaseServ 系统处理
type VerifyServ struct {
	ctx  *utils.Context
	dao  *verify.VerifyDAO
	cdao *customer.CustomerDAO
}

// NewVerifyServ 初始化
func NewVerifyServ(ctx *utils.Context) *VerifyServ {
	return &VerifyServ{
		ctx:  ctx,
		dao:  verify.NewVerifyDAO(ctx),
		cdao: customer.NewCustomerDAO(ctx),
	}
}

// GetCustomerCourseDetailById 根据ID获取当日可核销课程
func (s *VerifyServ) GetCustomerCourseDetailById(id, caseids string, page, pageSize int) (map[string]interface{}, error) {
	// if err := utils.NewAuthEngine(s.ctx).CheckCase(caseid); err != nil {
	// 	return nil, err
	// }
	if pageSize == 0 {
		pageSize = service.PAGENUM
	}
	if page == 0 {
		page = 1
	}
	// id, err := s.dao.GetCustomerCourseIdByQrcode(qrcode)
	// if err != nil {
	// 	utils.LogError("获取列表失败: " + err.Error())
	// 	return nil, errors.New("获取列表失败")
	// }
	customerDetail, err := s.dao.GetCustomerCourseList(id, caseids, page, pageSize)
	if err != nil {
		utils.LogError("获取列表失败: " + err.Error())
		return nil, errors.New("获取列表失败")
	}
	if len(customerDetail) == 0 {
		return nil, nil
	}
	customerId := customerDetail[0].CustomerId
	total, err := s.dao.GetCustomerCourseListCount(id, caseids)
	if err != nil {
		utils.LogError("获取列表失败: " + err.Error())
		return nil, errors.New("获取列表失败")
	}

	customer, err := s.cdao.GetCustomerByID(customerId)
	if err != nil {
		utils.LogError("获取列表失败: " + err.Error())
		return nil, errors.New("获取列表失败")
	}
	return map[string]interface{}{
		"list":         customerDetail,
		"pagesize":     pageSize,
		"pagenum":      total,
		"page":         page,
		"customerInfo": customer,
	}, nil
}

// GetCustomerCourseDetailByTel根据电话号码获取当日可核销课程
func (s *VerifyServ) GetCustomerCourseDetailByTel(tel, caseids string, page, pageSize int) (map[string]interface{}, error) {
	if pageSize == 0 {
		pageSize = service.PAGENUM
	}
	if page == 0 {
		page = 1
	}
	customerDetail, err := s.dao.GetCustomerCourseListByTel(tel, caseids, page, pageSize)
	if err != nil {
		utils.LogError("获取课程列表失败: " + err.Error())
		return nil, errors.New("获取课程列表失败")
	}
	if len(customerDetail) == 0 {
		return nil, errors.New("今日无可核销课程")
	}
	customerID := customerDetail[0].CustomerId
	total, err := s.dao.GetCustomerCourseListByTelCount(tel, caseids)
	if err != nil {
		utils.LogError("获取列表数量失败: " + err.Error())
		return nil, errors.New("获取列表数量失败")
	}
	customer, err := s.cdao.GetCustomerByID(customerID)
	if err != nil {
		utils.LogError("获取客户详情: " + err.Error())
		return nil, errors.New("获取客户详情")
	}
	return map[string]interface{}{
		"list":         customerDetail,
		"pagesize":     pageSize,
		"pagenum":      total,
		"page":         page,
		"customerInfo": customer,
	}, nil

}
func (s *VerifyServ) GetCustomerCourseDetailByDetailId(id string) (*model.TaCustomerCourseDetail, error) {
	customerDetail, err := s.dao.GetCustomerCourseDetailById(id)
	if err != nil {
		utils.LogError("获取核销记录失败: " + err.Error())
		return nil, errors.New("获取核销记录失败")
	}
	return customerDetail, err
}

// UpdateCustomerCourseDetail更新我的课程详情
func (s *VerifyServ) UpdateCustomerCourseDetail(customerDetail *model.TaCustomerCourseDetail, userid string) error {
	var err error
	customerDetail.VerifyDate = time.Now()
	customerDetail.VerifyUser = userid
	if time.Now().After(customerDetail.EndDate) {
		customerDetail.VerifyStatus = models.VERIFY_LATE
	} else {
		customerDetail.VerifyStatus = models.VERIFY_USED
	}
	err = s.dao.UpdateCustomerCourseDetail(customerDetail)
	if err != nil {
		utils.LogError("更新核销记录失败: " + err.Error())
		return errors.New("更新核销记录失败")
	}
	return err
}

// UpdateCustomerCard 更新我的体验卡
func (s *VerifyServ) UpdateCustomerCard(customerCard *model.TaCustomerCard) error {
	var err error
	customerCard.VerifyDate = time.Now()
	err = s.dao.UpdateCustomerCard(customerCard)
	if err != nil {
		utils.LogError("更新记录失败: " + err.Error())
		return errors.New("更新记录失败")
	}
	return nil

}

// UpdateCustomerCourse 更新我的课程
func (s *VerifyServ) UpdateCustomerCourse(customerCourse *model.TaCustomerCourse) error {
	var err error
	customerCourse.JoinNum = customerCourse.JoinNum + 1
	if customerCourse.JoinNum >= customerCourse.CourseNum {
		customerCourse.IsDone = models.BOOL_TRUE
	}
	err = s.dao.UpdateCustomerCourse(customerCourse)
	if err != nil {
		utils.LogError("更新记录失败: " + err.Error())
		return errors.New("更新记录失败")
	}
	return err

}

// GetCustomerCardByDetailId 根据我的课程详情id获取体验卡
func (s *VerifyServ) GetCustomerCardByDetailId(detailId string) (*model.TaCustomerCard, error) {
	customerCard, err := s.dao.GetCustomerCardByDetailId(detailId)
	if err != nil {
		utils.LogError("获取体验卡信息失败: " + err.Error())
		return nil, errors.New("获取体验卡信息失败")
	}
	return customerCard, err
}

// GetCustomerCourseByDetailId 根据我的课程详情id获取我的课程
func (s *VerifyServ) GetCustomerCourseByDetailId(detailId string) (*model.TaCustomerCourse, error) {
	customerCourse, err := s.dao.GetCustomerCourseByDetailId(detailId)
	if err != nil {
		utils.LogError("获取课程信息失败: " + err.Error())
		return nil, errors.New("获取课程信息失败")
	}
	return customerCourse, err
}
